@@ -3,6 +3,7 @@ from __future__ import annotations
 
 import json
 import logging
+from itertools import chain
 
 import voluptuous as vol
 
@@ -220,6 +221,9 @@ class TasmotaDiscovery:
                         return
                     self._devices[mac] = payload
                 else:
+                    if mac not in self._devices:
+                        return
+
                     self._devices.pop(mac, None)
                     payload = {}
 
@@ -264,17 +268,22 @@ class TasmotaDiscovery:
         }
         self._sub_state = await self._mqtt_client.subscribe(self._sub_state, topics)
 
-
-def clear_discovery_topic(
-    mac: str, discovery_prefix: str, mqtt_client: TasmotaMQTTClient
-) -> None:
-    """Clear retained discovery topic."""
-    mac = mac.replace(":", "")
-    mac = mac.upper()
-    device_discovery_topic = f"{discovery_prefix}/{mac}/config"
-    mqtt_client.publish(device_discovery_topic, "", retain=True)
-    sensor_discovery_topic = f"{discovery_prefix}/{mac}/sensors"
-    mqtt_client.publish(sensor_discovery_topic, "", retain=True)
+    async def clear_discovery_topic(self, mac: str, discovery_prefix: str) -> None:
+        """Clear retained discovery topic."""
+        mac = mac.replace(":", "")
+        mac = mac.upper()
+        device_discovery_topic = None
+        sensor_discovery_topic = None
+        if mac in self._devices:
+            device_discovery_topic = f"{discovery_prefix}/{mac}/config"
+            self._devices.pop(mac)
+        if mac in self._sensors:
+            sensor_discovery_topic = f"{discovery_prefix}/{mac}/sensors"
+            self._sensors.pop(mac)
+        if device_discovery_topic:
+            await self._mqtt_client.publish(device_discovery_topic, "", retain=True)
+        if sensor_discovery_topic:
+            await self._mqtt_client.publish(sensor_discovery_topic, "", retain=True)
 
 
 def get_device_config_helper(discovery_msg: dict) -> TasmotaDeviceConfig:
@@ -322,9 +331,26 @@ def get_cover_entities(
     relays = discovery_msg[CONF_RELAY]
     shutter_entities: list[tuple[TasmotaShutterConfig | None, DiscoveryHashType]] = []
     shutter_indices = []
-    for (idx, value) in enumerate(relays):
-        if value == RL_SHUTTER and idx and relays[idx - 1] == RL_SHUTTER:
-            shutter_indices.append(idx - 1)
+
+    # Tasmota supports up to 4 shutters, each shutter is assigned two consecutive relays
+    for (idx, value) in enumerate(chain(relays, [-1])):
+        if idx - 1 in shutter_indices:
+            # This is the 2nd half of a pair, skip
+            continue
+
+        if value == RL_SHUTTER:
+            if relays[idx + 1] == RL_SHUTTER:
+                shutter_indices.append(idx)
+                _LOGGER.debug("Found shutter pair %s + %s", idx, idx + 1)
+            else:
+                # The 2nd half of the pair is missing, abort
+                _LOGGER.error(
+                    "Invalid shutter configuration, relay %s is shutter but %s is not",
+                    idx + 1,
+                    idx + 2,
+                )
+                shutter_indices = []
+                break
 
     # pad / truncate the shutter index list to 4
     shutter_indices = shutter_indices[:4] + [-1] * (4 - len(shutter_indices))
